import os
import http
import json
import functools

from urllib import request
from urllib import error


from typing import Dict, Optional, Any

from . import exceptions


def event(name: str, text: str, **kwargs):
    return request.urlopen(
        request.Request(f"{os.environ['PPA_URL']}/v1/{name}", json.dumps(dict(text=text, **kwargs)).encode("utf-8"))
    )


def task_event(status_code_exceptions: Optional[Dict[int, Exception]] = None, return_tuple=None):
    def _catch_status_code(status_code: int):
        try:
            raise status_code_exceptions[status_code]
        except (TypeError, KeyError):
            pass

    def _wrapped_task_event(func):
        @functools.wraps(func)
        def _process_task_event(*args, **kwargs) -> Any:
            def _send_request(request_func) -> http.client.HTTPResponse:
                try:
                    return request_func()
                except error.HTTPError as e:
                    # If the function has been decorated with custom exceptions then we may be able to match one
                    # of the status codes here. Note: not every non-200 status code will raise a urllib exception.
                    _catch_status_code(e.code)
                    try:
                        # Nothing provided in the decorator has caught the issue yet, so raise
                        # generic 400 and 500 exceptions if appropriate.
                        raise {
                            400: exceptions.BadRequest(e.read().decode("utf-8")),
                            500: exceptions.PPAServerError(e.read().decode("utf-8")),
                        }[e.code] from e
                    except KeyError:
                        # The request exception isn't handled and will be re-raised below.
                        pass
                    raise e

            response = _send_request(functools.partial(func, *args, **kwargs))
            status_code = response.getcode()
            _catch_status_code(status_code)

            # Any response that made it this far has been deemed acceptable. We don't care
            # about the status code anymore, just return response_json (which can be None
            # if the event is a UI output).
            try:
                response_json = json.loads(response.read())
            except json.decoder.JSONDecodeError:
                return None

            return return_tuple(**response_json) if return_tuple else response_json

        return _process_task_event

    return _wrapped_task_event
