"""
Generate

This module contains actions used to generate various types of data.
"""

import uuid

from typing import List

from ppa import ui
from ppa_tools import generate


def regex_password(regex: str = r"[A-Za-z0-9\-£\:\$\#\%\&\^]", length: int = 24) -> str:
    r"""Generate a password based on a regular expression.

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __regex:__ a regular expression containing a set of characters (defaults to _[A-Za-z0-9\-£\:\$\#\%\&\^]_)

        - __length:__ the length of the generated password (defaults to _15_)

    ??? success "Output"
        The generated password as a string.

    ??? example

        Generating a 20 character password using the default character set.

        ```YAML linenums="1"
        - ppa_tools.generate.regex_password:
            length: 20
          save: generated_password
        ```
    """
    return generate.password_from_regex(regex=regex, length=length)


def dictionary_password(*, length: int = 15, word_count: int = 3, valid_word_lengths: List[int] = list(range(3, 9))) -> str:
    """Generate a password containing 1 or more dictionary words.

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __length:__ the length of the generated password (defaults to _15_)

        - __word_count:__ the number of words the password should consist of (defaults to _3_)

        - __valid_word_lengths:__ a list of valid word lengths (defaults to _3_-_8_)

    ??? success "Output"
        The generated password as a string.

    ??? example

        Generating a 20 character password consisting of 4 words.

        ```YAML linenums="1"
        - ppa_tools.generate.dictionary_password:
            length: 20
            word_count: 4
          save: generated_password
        ```
    """
    return generate.password_from_words(length=length, word_count=word_count, valid_word_lengths=valid_word_lengths).password


def dictionary_password_interactive(
    *, length: int = 15, word_count: int = 3, valid_word_lengths: List[int] = list(range(3, 9))
) -> str:
    """Interactively generate a password containing 1 or more dictionary words.

    Each time a password is generated, the Task Operator will be given the choice to:

    - _Use Password_
    - _Generate Another_

    The process can be repeated as many times as needed.

    !!! question "Minimum Plugin Version: 1.0.0"

    !!! danger "Remember"

        Each generated password will be displayed to the Task Operator in a markdown document.

    ??? info "Input"

        - __length:__ the length of the generated password (defaults to _15_)

        - __word_count:__ the number of words the password should consist of (defaults to _3_)

        - __valid_word_lengths:__ a list of valid word lengths (defaults to _3_-_8_)

    ??? success "Output"
        The generated password as a string.

    ??? example

        Starting an interactive password generation process using default values.

        ```YAML linenums="1"
        - ppa_tools.generate.dictionary_password_interactive:
          save: generated_password
        ```
    """
    markdown_id = str(uuid.uuid4())
    confirm_id = str(uuid.uuid4())
    while True:
        password = generate.password_from_words(length=length, word_count=word_count, valid_word_lengths=valid_word_lengths)
        title_cased_display = " ".join([word.title() for word in password.display.split(" ")])
        ui.output_markdown(
            f"""
<html>
    <body>
        <div style="font-weight: bold; float: left;">Password: </div>
        <div style="padding-left: 5px; font-style: bold; float: left;">{password.password}</div>
        <div style="float: left; padding-left: 10px;">(</div>
            <div style="color: orange; font-style: italic; float: left;">{title_cased_display}</div>
        <div style="float: left;">)</div>
    </body>
</html>
        """,
            element_id=markdown_id,
        )
        if ui.input_confirm("Please choose an option", confirm="Use Password", cancel="Generate Another", element_id=confirm_id):
            return password.password
